/*
 * File: korektor.c
 * Description: A wrapping for the Polish proof-reader
 * Author: Stanislaw Galus
 * Created: 28.X.2003
 * Revision: $Id: korektor.c,v 0.6 2003/12/30 09:32:45 sgalus Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef MCHECK
#include <mcheck.h>
#endif
#include "libppr.h"

#define PROGRAM_NAME "korektor"
#define PROGRAM_VERSION "0.2"
#define MAILING_ADDRESS "sgalus@poczta.onet.pl"

static int getopt(int argc, char *argv[]);

int main(int argc, char *argv[])
{
  int r;

#ifdef MCHECK
  mtrace();
#endif

  program_name = argv[0];
  r = getopt(argc, argv);
  if (r == 1)
  {
    r = pprbegin();
    if (r == 0)
      r = pprrun();
  }
  pprend();
  if (r)
    wprintf(" %s: %d\n", "program returned", r);
  return r ? EXIT_FAILURE : EXIT_SUCCESS;
}

/*
 * Depending on command-line options:
 * - print brief documentation or program version and return 0,
 * - collect options and return 1,
 * - print an error message and return 2.
 */
static int getopt(int argc, char *argv[])
{
  const int optlen = 2;         /* length of "-h", "-v" and the like */
  namelist **w = &wfns;
  namelist **f = &ifns;
  int i = 0;
  char *s;

  while (++i < argc)
  {
    if (strcmp(argv[i], "-h") == 0)
    {
      printf("Usage: %s [option]... [file]...\n", program_name);
      printf("Options:\n");
      printf("-h\tPrint this documentation and exit.\n");
      printf("-v\tPrint information about the program and exit.\n");
      printf("-e character-encoding\n\tAssume that input files use "
             "character-encoding. Character\n\tencoding may be "
             "\"ISO-8859-2\", \"cp852\" or \"windows-1250\". The\n"
             "\tdefault is ISO-8859-2.\n");
      printf("-m operating-mode\n\tSet operating-mode. Operating mode "
             "may be \"normal\", \"extended\"\n\tor \"build\". The "
             "default is normal.\n");
      printf("-o output-file\n\tWrite results to the output-file "
             "instead of standard output.\n");
      printf("-w word-file\n\tAdd words from the word-file to the "
             "dictionary.\n");
      printf("The character \"-\" used as a file name refers to "
             "standard input or\noutput. If no file is given, standard "
             "input is read.\n");
      printf("\nReport bugs to " MAILING_ADDRESS ".\n");
      return 0;
    }
    else if (strcmp(argv[i], "-v") == 0)
    {
      printf("%s %s\n", PROGRAM_NAME, PROGRAM_VERSION);
      return 0;
    }
    else if (strncmp(argv[i], "-e", optlen) == 0)
    {
      if (!*(s = argv[i] + optlen))
      {
        if (++i >= argc)
        {
          wprintf(" %s\n", "missing character encoding");
          return 2;
        }
        s = argv[i];
      }
      if (strcmp(s, "ISO-8859-2") == 0)
        encoding = ISO88592;
      else if (strcmp(s, "cp852") == 0)
        encoding = CP852;
      else if (strcmp(s, "windows-1250") == 0)
        encoding = WINDOWS1250;
      else
      {
        wprintf(" %s: %s\n", "invalid character encoding", s);
        return 2;
      }
    }
    else if (strncmp(argv[i], "-m", optlen) == 0)
    {
      if (!*(s = argv[i] + optlen))
      {
        if (++i >= argc)
        {
          wprintf(" %s\n", "missing operating mode");
          return 2;
        }
        s = argv[i];
      }
      if (strcmp(s, "normal") == 0)
        mode = NORMAL;
      else if (strcmp(s, "extended") == 0)
        mode = EXTENDED;
      else if (strcmp(s, "build") == 0)
        mode = BUILDWF;
      else
      {
        wprintf(" %s: %s\n", "invalid operating mode", s);
        return 2;
      }
    }
    else if (strncmp(argv[i], "-o", optlen) == 0)
    {
      if (!*(s = argv[i] + optlen))
      {
        if (++i >= argc)
        {
          wprintf(" %s\n", "missing output file name");
          return 2;
        }
        s = argv[i];
      }
      outfn = s;
    }
    else if (strncmp(argv[i], "-w", optlen) == 0)
    {
      if (!*(s = argv[i] + optlen))
      {
        if (++i >= argc)
        {
          wprintf(" %s\n", "missing word file name");
          return 2;
        }
        s = argv[i];
      }
      *w = (namelist *)malloc(sizeof(namelist));
      if (*w == NULL)
      {
        wprintf(" %s\n", "too many word files");
        return 2;
      }
      (*w)->name = s;
      (*w)->next = NULL;
      w = &(*w)->next;
    }
    else                        /* input file name */
    {
      *f = (namelist *)malloc(sizeof(namelist));
      if (*f == NULL)
      {
        wprintf(" %s\n", "too many input files");
        return 2;
      }
      (*f)->name = argv[i];
      (*f)->next = NULL;
      f = &(*f)->next;
    }
  }
  if (ifns == NULL)
  {
    ifns = (namelist *)malloc(sizeof(namelist));
    if (ifns == NULL)
    {
      wprintf(" %s\n", "out of memory");
      return 2;
    }
    ifns->name = STDIOS;
    ifns->next = NULL;
  }
  return 1;
}
